@@ -1,4 +1,4 @@
-from robotoff.types import JSONType
+from robotoff.types import JSONType, NeuralCategoryClassifierModel
 
 IMAGE_PREDICTION_IMPORTER_SCHEMA: JSONType = {
     "$schema": "http://json-schema.org/draft-07/schema#",
@@ -60,8 +60,13 @@ PREDICT_CATEGORY_SCHEMA: JSONType = {
                     "type": "array",
                     "items": {"enum": ["neural", "matcher"]},
                 },
+                "neural_model_name": {
+                    "type": "string",
+                    "enum": [x.name for x in NeuralCategoryClassifierModel],
+                },
             },
             "required": ["barcode"],
+            "additionalProperties": False,
         },
         {
             "type": "object",
@@ -77,8 +82,28 @@ PREDICT_CATEGORY_SCHEMA: JSONType = {
                             "type": "array",
                             "items": {"type": "string"},
                         },
+                        "ocr": {
+                            "type": "array",
+                            "items": {"type": "string"},
+                        },
+                        "nutriments": {
+                            "type": "object",
+                            "properties": {
+                                "fat_100g": {"type": "number"},
+                                "saturated-fat_100g": {"type": "number"},
+                                "carbohydrates_100g": {"type": "number"},
+                                "sugars_100g": {"type": "number"},
+                                "fiber_100g": {"type": "number"},
+                                "proteins_100g": {"type": "number"},
+                                "salt_100g": {"type": "number"},
+                                "energy-kcal_100g": {"type": "number"},
+                                "fruits-vegetables-nuts_100g": {"type": "number"},
+                            },
+                            "additionalProperties": False,
+                        },
                     },
-                    "required": ["product_name"],
+                    "required": [],
+                    "additionalProperties": False,
                 },
                 "deepest_only": {
                     "type": "boolean",
@@ -92,8 +117,13 @@ PREDICT_CATEGORY_SCHEMA: JSONType = {
                     "type": "string",
                     "minLength": 1,
                 },
+                "neural_model_name": {
+                    "type": "string",
+                    "enum": [x.name for x in NeuralCategoryClassifierModel],
+                },
             },
             "required": ["product"],
+            "additionalProperties": False,
         },
     ],
 }
