@@ -7,7 +7,7 @@ from robotoff.taxonomy import get_taxonomy
 from robotoff.types import PredictionType
 from robotoff.utils import get_logger
 
-from .dataclass import OCRField, OCRRegex, OCRResult, get_text
+from .dataclass import OCRField, OCRRegex, OCRResult, get_match_bounding_box, get_text
 
 logger = get_logger(__name__)
 
@@ -35,46 +35,42 @@ AOC_REGEX = {
         OCRRegex(
             # re.compile(r"(?<=appellation\s).*(?=(\scontr[ôo]l[ée]e)|(\sprot[ée]g[ée]e))"),
             re.compile(
-                r"(appellation)\s*(?P<category>.+)\s*(contr[ôo]l[ée]e|prot[ée]g[ée]e)"
+                r"(appellation)\s*(?P<category>.+)\s*(contr[ôo]l[ée]e|prot[ée]g[ée]e)",
+                re.I,
             ),
             field=OCRField.full_text_contiguous,
-            lowercase=True,
             processing_func=category_taxonomisation,
         ),
         OCRRegex(
             re.compile(
-                r"(?P<category>.+)\s*(appellation d'origine contr[ôo]l[ée]e|appellation d'origine prot[ée]g[ée]e)"
+                r"(?P<category>.+)\s*(appellation d'origine contr[ôo]l[ée]e|appellation d'origine prot[ée]g[ée]e)",
+                re.I,
             ),
             field=OCRField.full_text_contiguous,
-            lowercase=True,
             processing_func=category_taxonomisation,
         ),
     ],
     "es:": [
         OCRRegex(
-            re.compile(r"(?P<category>.+)(\s*denominacion de origen protegida)"),
+            re.compile(r"(?P<category>.+)(\s*denominacion de origen protegida)", re.I),
             field=OCRField.full_text_contiguous,
-            lowercase=True,
             processing_func=category_taxonomisation,
         ),
         OCRRegex(
-            re.compile(r"(denominacion de origen protegida\s*)(?P<category>.+)"),
+            re.compile(r"(denominacion de origen protegida\s*)(?P<category>.+)", re.I),
             field=OCRField.full_text_contiguous,
-            lowercase=True,
             processing_func=category_taxonomisation,
         ),
     ],
     "en:": [
         OCRRegex(
-            re.compile(r"(?P<category>.+)\s*(aop|dop|pdo)"),
+            re.compile(r"(?P<category>.+)\s*(aop|dop|pdo)", re.I),
             field=OCRField.full_text_contiguous,
-            lowercase=True,
             processing_func=category_taxonomisation,
         ),
         OCRRegex(
-            re.compile(r"(aop|dop|pdo)\s*(?P<category>.+)"),
+            re.compile(r"(aop|dop|pdo)\s*(?P<category>.+)", re.I),
             field=OCRField.full_text_contiguous,
-            lowercase=True,
             processing_func=category_taxonomisation,
         ),
     ],
@@ -98,19 +94,26 @@ def find_category(content: Union[OCRResult, str]) -> list[Prediction]:
                 continue
 
             for match in ocr_regex.regex.finditer(text):
-
                 if ocr_regex.processing_func:
                     category_value = ocr_regex.processing_func(lang, match)
 
                 if category_value is None:
                     continue
 
+                data = {"text": match.group(), "notify": ocr_regex.notify}
+                if (
+                    bounding_box := get_match_bounding_box(
+                        content, match.start(), match.end()
+                    )
+                ) is not None:
+                    data["bounding_box_absolute"] = bounding_box
+
                 predictions.append(
                     Prediction(
                         type=PredictionType.category,
                         value_tag=category_value,
                         predictor="regex",
-                        data={"text": match.group(), "notify": ocr_regex.notify},
+                        data=data,
                         automatic_processing=False,
                     )
                 )
