@@ -9,7 +9,7 @@ from robotoff.types import PredictionType
 from robotoff.utils import get_logger, text_file_iter
 from robotoff.utils.cache import CachedStore
 
-from .dataclass import OCRField, OCRRegex, OCRResult, get_text
+from .dataclass import OCRField, OCRRegex, OCRResult, get_match_bounding_box, get_text
 from .utils import generate_keyword_processor
 
 logger = get_logger(__name__)
@@ -41,9 +41,10 @@ EN_ORGANIC_REGEX_STR = [
 LABELS_REGEX = {
     "en:organic": [
         OCRRegex(
-            re.compile(r"|".join([r"(?:{})".format(x) for x in EN_ORGANIC_REGEX_STR])),
+            re.compile(
+                r"|".join([r"(?:{})".format(x) for x in EN_ORGANIC_REGEX_STR]), re.I
+            ),
             field=OCRField.full_text_contiguous,
-            lowercase=True,
         ),
     ],
     "xx-bio-xx": [
@@ -53,144 +54,130 @@ LABELS_REGEX = {
             re.compile(
                 r"(?<![a-zA-Z])([A-Z]{2})[\-\s.](BIO|ÖKO|OKO|EKO|ØKO|ORG|Bio)[\-\s.](\d{2,3})"
             ),
-            field=OCRField.text_annotations,
-            lowercase=False,
+            field=OCRField.full_text,
             processing_func=process_eu_bio_label_code,
         ),
         # Spain specific regex
         OCRRegex(
             re.compile(r"(?<![a-zA-Z])ES[\-\s.]ECO[\-\s.](\d{3})[\-\s.]([A-Z]{2,3})"),
-            field=OCRField.text_annotations,
-            lowercase=False,
+            field=OCRField.full_text,
             processing_func=process_es_bio_label_code,
         ),
     ],
     "fr:ab-agriculture-biologique": [
         OCRRegex(
-            re.compile(r"certifi[ée] ab[\s.,)]"),
+            re.compile(r"certifi[ée] ab[\s.,)]", re.I),
             field=OCRField.full_text_contiguous,
-            lowercase=True,
         ),
     ],
     "en:pgi": [
         OCRRegex(
             re.compile(
-                r"indication g[ée]ographique prot[eé]g[eé]e|Indicazione geografica protetta|geschützte geografische angabe"
+                r"indication g[ée]ographique prot[eé]g[eé]e|Indicazione geografica protetta|geschützte geografische angabe",
+                re.I,
             ),
             field=OCRField.full_text_contiguous,
-            lowercase=True,
         ),
         OCRRegex(
             re.compile(r"(?<!\w)(?:IGP|BGA|PGI)(?!\w)"),
             field=OCRField.full_text_contiguous,
-            lowercase=False,
         ),
     ],
     "fr:label-rouge": [
         OCRRegex(
-            re.compile(r"d[ée]cret du 0?5[./]01[./]07"),
+            re.compile(r"d[ée]cret du 0?5[./]01[./]07", re.I),
             field=OCRField.full_text_contiguous,
-            lowercase=True,
         ),
         OCRRegex(
-            re.compile(r"(?<!\w)homologation(?: n°?)? ?la ?\d{2}\/\d{2}(?!\w)"),
+            re.compile(r"(?<!\w)homologation(?: n°?)? ?la ?\d{2}\/\d{2}(?!\w)", re.I),
             field=OCRField.full_text_contiguous,
-            lowercase=True,
         ),
     ],
     "en:pdo": [
         OCRRegex(
             re.compile(r"(?<!\w)(?:PDO|AOP|DOP)(?!\w)"),
             field=OCRField.full_text_contiguous,
-            lowercase=False,
         ),
         OCRRegex(
-            re.compile(r"appellation d'origine prot[eé]g[eé]e"),
+            re.compile(r"appellation d'origine prot[eé]g[eé]e", re.I),
             field=OCRField.full_text_contiguous,
-            lowercase=True,
         ),
     ],
     "fr:aoc": [
         OCRRegex(
-            re.compile(r"(?<!\w)(?:AOC)(?!\w)"),
-            field=OCRField.full_text_contiguous,
-            lowercase=False,
+            re.compile(r"(?<!\w)(?:AOC)(?!\w)"), field=OCRField.full_text_contiguous
         ),
     ],
     "en:nutriscore": [
-        OCRRegex(
-            re.compile(r"NUTRI-SCORE"),
-            field=OCRField.full_text,
-            lowercase=False,
-        ),
+        OCRRegex(re.compile(r"NUTRI-SCORE"), field=OCRField.full_text),
     ],
     "en:eu-non-eu-agriculture": [
         OCRRegex(
             re.compile(
-                r"agriculture ue\s?/\s?non\s?(?:-\s?)?ue|eu\s?/\s?non\s?(?:-\s?)?eu agriculture"
+                r"agriculture ue\s?/\s?non\s?(?:-\s?)?ue|eu\s?/\s?non\s?(?:-\s?)?eu agriculture",
+                re.I,
             ),
             field=OCRField.full_text_contiguous,
-            lowercase=True,
         ),
     ],
     "en:eu-agriculture": [
         # The negative lookafter/lookbehind forbid matching "agriculture ue/non ue"
         OCRRegex(
-            re.compile(r"agriculture ue(?!\s?/)|(?<!-)\s?eu agriculture"),
+            re.compile(r"agriculture ue(?!\s?/)|(?<!-)\s?eu agriculture", re.I),
             field=OCRField.full_text_contiguous,
-            lowercase=True,
         ),
     ],
     "en:non-eu-agriculture": [
         OCRRegex(
-            re.compile(r"agriculture non\s?(?:-\s?)?ue|non\s?(?:-\s?)?eu agriculture"),
+            re.compile(
+                r"agriculture non\s?(?:-\s?)?ue|non\s?(?:-\s?)?eu agriculture", re.I
+            ),
             field=OCRField.full_text_contiguous,
-            lowercase=True,
         ),
     ],
     "en:no-preservatives": [
         OCRRegex(
             re.compile(
-                r"senza conservanti(?! arti)|без консервантов|conserveermiddelvrij|(?<!\w)(?:sans|ni) conservateur(?!s? arti)|fără conservanți|no preservative|sin conservante(?!s? arti)|ohne konservierungsstoffe"
+                r"senza conservanti(?! arti)|без консервантов|conserveermiddelvrij|(?<!\w)(?:sans|ni) conservateur(?!s? arti)|fără conservanți|no preservative|sin conservante(?!s? arti)|ohne konservierungsstoffe",
+                re.I,
             ),
             field=OCRField.full_text_contiguous,
-            lowercase=True,
         ),
     ],
     "en:no-flavors": [
         OCRRegex(
             re.compile(
-                r"без ароматизаторов|senza aromi|zonder toegevoegde smaakstoffen|(?<!\w)(?:sans|ni) ar[ôo]mes? ajout[ée]s|sin aromas?|ohne zusatz von aromen|no flavors?"
+                r"без ароматизаторов|senza aromi|zonder toegevoegde smaakstoffen|(?<!\w)(?:sans|ni) ar[ôo]mes? ajout[ée]s|sin aromas?|ohne zusatz von aromen|no flavors?",
+                re.I,
             ),
             field=OCRField.full_text_contiguous,
-            lowercase=True,
         ),
     ],
     "en:no-artificial-flavors": [
         OCRRegex(
             re.compile(
-                r"без искусственных ароматизаторов|ohne künstliche aromen|sin aromas? artificiales?|vrij van kunstmatige smaakstoffen|(?<!\w)(?:sans|ni) ar[ôo]mes? artificiels?|no artificial flavors?"
+                r"без искусственных ароматизаторов|ohne künstliche aromen|sin aromas? artificiales?|vrij van kunstmatige smaakstoffen|(?<!\w)(?:sans|ni) ar[ôo]mes? artificiels?|no artificial flavors?",
+                re.I,
             ),
             field=OCRField.full_text_contiguous,
-            lowercase=True,
         ),
     ],
     "en:no-colorings": [
         OCRRegex(
             re.compile(
-                r"no colorings?|no colourants?|ohne farbstoffzusatz|(?<!\w)(?:sans|ni) colorants?|zonder kleurstoffen|sin colorantes?|без красителей|senza coloranti"
+                r"no colorings?|no colourants?|ohne farbstoffzusatz|(?<!\w)(?:sans|ni) colorants?|zonder kleurstoffen|sin colorantes?|без красителей|senza coloranti",
+                re.I,
             ),
             field=OCRField.full_text_contiguous,
-            lowercase=True,
         ),
     ],
     "en:no-additives": [
         OCRRegex(
             re.compile(
-                r"zonder toevoegingen|sin aditivos(?! arti)|(?<!\w)(?:sans|ni) additif(?!s? arti)|ohne zusätze|no additives?"
+                r"zonder toevoegingen|sin aditivos(?! arti)|(?<!\w)(?:sans|ni) additif(?!s? arti)|ohne zusätze|no additives?",
+                re.I,
             ),
             field=OCRField.full_text_contiguous,
-            lowercase=True,
         ),
     ],
 }
@@ -218,20 +205,30 @@ def generate_label_keyword_processor(labels: Optional[Iterable[str]] = None):
     return generate_keyword_processor(labels)
 
 
-def extract_label_flashtext(processor: KeywordProcessor, text: str) -> list[Prediction]:
+def extract_label_flashtext(
+    processor: KeywordProcessor, content: Union[OCRResult, str]
+) -> list[Prediction]:
     predictions = []
 
+    text = get_text(content)
     for (label_tag, _), span_start, span_end in processor.extract_keywords(
         text, span_info=True
     ):
         match_str = text[span_start:span_end]
+        data = {"text": match_str, "notify": False}
+
+        if (
+            bounding_box := get_match_bounding_box(content, span_start, span_end)
+        ) is not None:
+            data["bounding_box_absolute"] = bounding_box
+
         predictions.append(
             Prediction(
                 type=PredictionType.label,
                 value_tag=label_tag,
                 automatic_processing=False,
                 predictor="flashtext",
-                data={"text": match_str, "notify": False},
+                data=data,
             )
         )
 
@@ -264,19 +261,25 @@ def find_labels(content: Union[OCRResult, str]) -> list[Prediction]:
                 else:
                     label_value = label_tag
 
+                data = {"text": match.group(), "notify": ocr_regex.notify}
+                if (
+                    bounding_box := get_match_bounding_box(
+                        content, match.start(), match.end()
+                    )
+                ) is not None:
+                    data["bounding_box_absolute"] = bounding_box
+
                 predictions.append(
                     Prediction(
                         type=PredictionType.label,
                         value_tag=label_value,
                         predictor="regex",
-                        data={"text": match.group(), "notify": ocr_regex.notify},
+                        data=data,
                     )
                 )
 
     processor = LABEL_KEYWORD_PROCESSOR_STORE.get()
-
-    text = get_text(content)
-    predictions += extract_label_flashtext(processor, text)
+    predictions += extract_label_flashtext(processor, content)
 
     if isinstance(content, OCRResult):
         for logo_annotation in content.logo_annotations:
