@@ -6,15 +6,15 @@ import spacy
 
 from robotoff.utils import get_logger
 
-from .fold_to_ascii import fold
+from .fold_to_ascii import fold, fold_without_insertion_deletion
 
 logger = get_logger(__name__)
 
 CONSECUTIVE_SPACES_REGEX = re.compile(r" {2,}")
 
 
-def strip_accents_ascii(s: str) -> str:
-    """Transform accentuated unicode symbols into ascii or nothing
+def strip_accents_v1(s: str) -> str:
+    """Transform accentuated unicode symbols into ascii or nothing.
 
     Warning: this solution is only suited for languages that have a direct
     transliteration to ASCII symbols.
@@ -33,8 +33,19 @@ def strip_accents_ascii(s: str) -> str:
     return nkfd_form.encode("ASCII", "ignore").decode("ASCII")
 
 
-def strip_accents_ascii_v2(s):
-    return fold(s)
+def strip_accents_v2(s: str, keep_length: bool = False) -> str:
+    """Strip accents and normalize string.
+
+    :param s: the string to normalize
+    :param keep_length: if True, no character is replaced without a
+        subtitution of length 1: the length of the string is therefore kept
+        unchanged. Default to False.
+    :return: the normalized string
+    """
+    if keep_length:
+        return fold_without_insertion_deletion(s)
+    else:
+        return fold(s)
 
 
 def strip_consecutive_spaces(text: str) -> str:
@@ -69,7 +80,7 @@ def get_tag(text: str) -> str:
     - replacement of punctuation by either a comma ("-") or nothing, depending
     on the punctuation
     """
-    text = strip_accents_ascii_v2(text)
+    text = strip_accents_v2(text)
     text = (
         text.lower()
         .replace(" & ", "-")
